/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: RuleManagerPM.as 4876 2009-08-11 14:22:08Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.manager.model.presentation
{
import flash.events.Event;
import flash.events.EventDispatcher;

import mx.binding.utils.*;
import mx.controls.Alert;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;

import org.ow2.jasmine.monitoring.eos.rules.common.event.SynchronizedRulePackageEvent;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackageState;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.AbstractRulePackageVO;
import org.ow2.jasmine.monitoring.eos.rules.manager.event.*;
import org.ow2.jasmine.monitoring.eos.rules.manager.model.domain.RuleManagerModel;
    
/**
* Presentation model for the rule manager view
* 
* @author Julien Nicoulaud
*/
[Bindable]
public class RuleManagerPM extends EventDispatcher
{
	
    // =========================================================================
    // Variables
    // =========================================================================
    
    /**
    * View states names
    */
    public static var UNSAVED_MODIFICATIONS : String = "unsavedModifications";
    
	/**
	* Domain model
	*/
	public var ruleManagerModel : RuleManagerModel;
    
    /**
    * The current wiew presentation state
    */
    public var presentationState : String = null;
    
    
    // =========================================================================
    // Constructors
    // =========================================================================
    
    /**
    * Build the Presentation Model for a rule manager view
    */
	public function RuleManagerPM(ruleManagerModel:RuleManagerModel)
	{
		// Set the model watched
		this.ruleManagerModel = ruleManagerModel;
		
		// Watch rule packages collection changes
        ruleManagerModel.rulePackages.addEventListener(CollectionEvent.COLLECTION_CHANGE,
                                                       rulePackagesCollectionChangeHandler);
                                      
        // Watch current "Save" operations pending count
        ChangeWatcher.watch(ruleManagerModel,
                            "pendingSaveOperations",
                            pendingSaveOperationsChange);
	}
    
    
    // =========================================================================
    // Public methods - API provided to the views
    // =========================================================================
	
	/**
	* Get the rule packages
	*/
	public function getRulesPackages():void
	{
		new RuleManagerEvent(RuleManagerEvent.GET_RULE_PACKAGES + ruleManagerModel.componentID).dispatch();
	}
    
    /**
    * Apply the changes made to the rule packages
    */
    public function applyChanges():void
    {
    	
    	presentationState = null;
    	
    	// Save each modified package
        for each (var rulePackage:SynchronizedRulePackage in ruleManagerModel.rulePackages) {
            if (rulePackage.state == SynchronizedRulePackageState.MODIFIED) {
                
                // Check issues
                var issues : String = rulePackage.rulePackage.canBeSaved();
                
                if (issues == null) {
                    
                    // Increment the number of pending operations
                    ruleManagerModel.pendingSaveOperations++;
                
                    // Send a save command event
                    new SynchronizedRulePackageEvent(SynchronizedRulePackageEvent.SAVE_SYNCHRONIZED_RULE_PACKAGE + ruleManagerModel.componentID,
                                                     rulePackage).dispatch();
                 
                 } else {
                    var cannotSaveAlert : Alert = Alert.show('You must fix the following problems before saving the rule package \"'
                                                             + rulePackage.rulePackage.displayName + '\":' + issues,
                                                             'The package cannot be saved',
                                                             4,
                                                             null,
                                                             null,
                                                             null,
                                                             4);
                }

            }
        }
        
        updatePresentationState();

    }
    
    /**
    * Activate a rule package
    */
    public function activateRulePackage(rulePackage:SynchronizedRulePackage):void
    {
    	rulePackage.state = SynchronizedRulePackageState.MODIFIED;
    	rulePackage.rulePackage.state = AbstractRulePackageVO.ACTIVATION_REQUESTED;
    }
    
    /**
    * Delete a rule package
    */
    public function deleteRulePackage(rulePackage:SynchronizedRulePackage):void
    {
        // Send a delete command event
        new SynchronizedRulePackageEvent(SynchronizedRulePackageEvent.DELETE_RULE_PACKAGE + ruleManagerModel.componentID,
                                         rulePackage).dispatch();
    }
    
    
    // =========================================================================
    // Domain model watching
    // =========================================================================
    
    /**
    * Each time the rule packages collection is updated
    */
    private function rulePackagesCollectionChangeHandler(event:CollectionEvent):void
    {
        switch (event.kind) {
            case CollectionEventKind.ADD:
            case CollectionEventKind.REFRESH:
            case CollectionEventKind.REPLACE:
            case CollectionEventKind.RESET:
            case CollectionEventKind.UPDATE:
                updatePresentationState();
                break;
        }
    }
    
    /**
    * Each time the  "Save" operations pending count changes
    */
    private function pendingSaveOperationsChange(e:Event=null):void
    {
        if (ruleManagerModel.pendingSaveOperations == 0) {
            updatePresentationState();
        }
    }
    
            
    // =========================================================================
    // Private methods
    // =========================================================================
    
    /**
    * Update the presentation state
    */
    private function updatePresentationState(e:Event=null):void
    {
    	// If no save operations pending
    	if (ruleManagerModel.pendingSaveOperations == 0) {
    		
    		// Check each package state
            for each (var rulePackage:SynchronizedRulePackage in ruleManagerModel.rulePackages) {
                if (rulePackage.state == SynchronizedRulePackageState.MODIFIED) {
                    presentationState = UNSAVED_MODIFICATIONS;
                    return;
                }
            }
        }
        presentationState = null;
    }

}
}
